/******************************************************************************
    Copyright (c) 1996-2000 Synopsys, Inc.    ALL RIGHTS RESERVED

  The contents of this file are subject to the restrictions and limitations
  set forth in the SYNOPSYS Open Source License Version 1.0  (the "License"); 
  you may not use this file except in compliance with such restrictions 
  and limitations. You may obtain instructions on how to receive a copy of 
  the License at

  http://www.synopsys.com/partners/tapin/tapinprogram.html. 

  Software distributed by Original Contributor under the License is 
  distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either 
  expressed or implied. See the License for the specific language governing 
  rights and limitations under the License.

******************************************************************************/

#include <stdio.h>
#include "mymalloc.h"
#include "LibParse.h"
#include "LibGroup.h"
#include "LibLoadMsg.h"
#include "UFile"
#include "UMsg"

// initialization of static members

LibGroup*           LibParse::d_group      = NULL;
LibDefineHashMap*   LibParse::d_defineHash = NULL;
LibMsgHandlerT      LibParse::d_msgHandler = LibDefaultMsgHandler;
uint_t              LibParse::d_debugMode  = 0;
const char*         LibParse::d_filename   = NULL;
const UControl*     LibParse::d_control    = NULL;

extern int liberty_parser_parse();
extern void liberty_parser2_restart(FILE *fp);
extern LibErrorT syntax_check(LibGroup* lib);

static const char*
LibGetErrorText(LibErrorT errorCode)
{
  switch (errorCode) {
  case LIB_NO_ERROR:
    return "No error Occurred!";
  case LIB_INTERNAL_SYSTEM_ERROR:
    return "Internal System Error";
  case LIB_SYNTAX_ERROR:
    return "Syntax Error Encountered";
  case LIB_INVALID_VALUE:
    return "Invalid Value";
  case LIB_INVALID_NAME:
    return "Invalid Name";
  case LIB_INVALID_OBJECTTYPE:
    return "Invalid Object Type";
  case LIB_INVALID_ATTRTYPE:
    return "Invalid Attribute Type";
  case LIB_UNUSABLE_OID:
    return "Oid Not Usable. Did you run PIInit?";
  case LIB_OBJECT_ALREADY_EXISTS:
    return "Object Name Already Being Used.";
  case LIB_OBJECT_NOT_FOUND:
    return "Object Not Found";
  case LIB_TRACE_FILES_CANNOT_BE_OPENED:
    return "Trace Files cannot be opened.";
  case LIB_PIINIT_NOT_CALLED:
    return "PIInit() not yet called.";
  case LIB_SEMANTIC_ERROR:
    return "Semantic Error found in data.";
  case LIB_REFERENCE_ERROR:
    return "Reference Error found in data.";
  case LIB_MAX_ERROR:
    return "?";
  }
  return "?";
}

void
LibParse::LibDefaultMsgHandler(LibSeverityT sev,
			 LibErrorT errToPrint,
			 const char* auxText)
{
  const char *errt = LibGetErrorText(errToPrint);

  switch (sev) {
  case LIB_SEVERITY_NOTE:
    LIBLOADMSG(parseInfo, auxText);
    break;
  case LIB_SEVERITY_WARN:
    LIBLOADMSG(parseWarn, auxText);
    break;
  case LIB_SEVERITY_ERR:
    if (auxText) {
      LIBLOADMSG(parseErrAux, auxText, errt);
    } else {
      LIBLOADMSG(parseErr, errt);
    }
    break;
  }
}


void
LibParse::init(const UControl& control)
{
  UString strNull;
  LibFileInfo fNull;

  liberty_pool_init();
  d_group = new IN_LIBERTY_POOL LibGroup(strNull, strNull, 0,
		         NULL, fNull);
  d_defineHash = new IN_LIBERTY_POOL LibDefineHashMap(0);
  d_msgHandler = LibDefaultMsgHandler;
  d_control = &control;
}

void
LibParse::quit()
{
  delete d_group;
  delete d_defineHash;
  d_group = NULL;
  d_defineHash = NULL;

  // my_malloc_print_totals();
  liberty_pool_free();
}

#ifdef RT_FPGA

extern void *(*createFpgaStream)(const char *);
extern int (*readFpgaStream)(void *, char *, int n);
extern void (*destroyFpgaStream)(void *);

#endif

LibErrorT
LibParse::read(const char *filename)
{
  extern FILE *liberty_parser2_in;
  extern int syntax_errors;

  d_filename = filename;
#ifdef RT_FPGA
  liberty_parser2_in = (FILE*)createFpgaStream(filename);
#else
  liberty_parser2_in = UFile::open(filename, "r");
#endif

  if (liberty_parser2_in == (FILE *) NULL) {
    return LIB_INVALID_NAME;
  }

  liberty_parser2_restart(liberty_parser2_in);
  liberty_parser_parse();

#ifdef RT_FPGA
  destroyFpgaStream(liberty_parser2_in);
#else
  UFile::close(liberty_parser2_in);
#endif

  if (syntax_errors)
    return LIB_SYNTAX_ERROR;
  else
    return LIB_NO_ERROR;
}

void
LibParse::write(const char *filename, LibGroup* group)
{
  /* open the file */
  FILE *of;

  of = UFile::open(filename, "w");
  if (!of) {
    return;
  }

  /* call the top level routine */
  group->dump(of, "");
  UFile::close(of);
}

LibErrorT
LibParse::checkSyntax(LibGroup* group)
{
  return (syntax_check(group));
}
